@@ -93,11 +93,13 @@ struct FMTreeViewDetails {
 	GtkWidget *popup_open_in_new_window;
 	GtkWidget *popup_open_in_new_tab;
 	GtkWidget *popup_create_folder;
+    GtkWidget *popup_post_create_folder_separator;
 	GtkWidget *popup_cut;
 	GtkWidget *popup_copy;
 	GtkWidget *popup_paste;
 	GtkWidget *popup_rename;
     GtkWidget *popup_pin;
+    GtkWidget *popup_post_pin_separator;
     GtkWidget *popup_unpin;
 	GtkWidget *popup_trash;
 	GtkWidget *popup_delete;
@@ -116,6 +118,7 @@ struct FMTreeViewDetails {
     GList *action_items;
     guint hidden_files_changed_id;
     guint sort_directories_first : 1;
+    guint sort_favorites_first : 1;
 };
 
 typedef struct {
@@ -543,7 +546,9 @@ compare_rows (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer call
         result = nemo_file_compare_for_sort (file_a, file_b,
                                              NEMO_FILE_SORT_BY_DISPLAY_NAME,
                                              FM_TREE_VIEW (callback_data)->details->sort_directories_first,
-                                             FALSE);
+                                             FM_TREE_VIEW (callback_data)->details->sort_favorites_first,
+                                             FALSE,
+                                             NULL);
 	}
 
 	nemo_file_unref (file_a);
@@ -689,13 +694,14 @@ static gboolean
 button_pressed_callback (GtkTreeView *treeview, GdkEventButton *event,
 			 FMTreeView *view)
 {
-	GtkTreePath *path, *cursor_path;
+	GtkTreePath *path;
 	gboolean parent_file_is_writable;
 	gboolean file_is_home_or_desktop;
 	gboolean file_is_special_link;
 	gboolean can_move_file_to_trash;
 	gboolean can_delete_file;
 	gboolean using_browser;
+    gint is_toplevel;
 
 	using_browser = g_settings_get_boolean (nemo_preferences,
 						NEMO_PREFERENCES_ALWAYS_USE_BROWSER);
@@ -719,12 +725,11 @@ button_pressed_callback (GtkTreeView *treeview, GdkEventButton *event,
 			gtk_tree_path_free (path);
 			return FALSE;
 		}
-		gtk_tree_view_get_cursor (view->details->tree_widget, &cursor_path, NULL);
-
-		gtk_tree_path_free (path);
 
 		create_popup_menu (view);
 
+        is_toplevel = gtk_tree_path_get_depth (path) == 1;
+
 		if (using_browser) {
 			gtk_widget_set_sensitive (view->details->popup_open_in_new_window,
 						  nemo_file_is_directory (view->details->popup_file));
@@ -735,6 +740,15 @@ button_pressed_callback (GtkTreeView *treeview, GdkEventButton *event,
 		gtk_widget_set_sensitive (view->details->popup_create_folder,
 			nemo_file_is_directory (view->details->popup_file) &&
 			nemo_file_can_write (view->details->popup_file));
+
+        if (nemo_file_is_in_favorites (view->details->popup_file)) {
+            gtk_widget_hide (view->details->popup_create_folder);
+            gtk_widget_hide (view->details->popup_post_create_folder_separator);
+        } else {
+            gtk_widget_show (view->details->popup_create_folder);
+            gtk_widget_show (view->details->popup_post_create_folder_separator);
+        }
+
 		gtk_widget_set_sensitive (view->details->popup_paste, FALSE);
 		if (nemo_file_is_directory (view->details->popup_file) &&
 			nemo_file_can_write (view->details->popup_file)) {
@@ -785,12 +799,20 @@ button_pressed_callback (GtkTreeView *treeview, GdkEventButton *event,
 			gtk_widget_hide (view->details->popup_unmount_separator);
 		}
 
-        if (nemo_file_get_pinning (view->details->popup_file)) {
-            gtk_widget_hide (view->details->popup_pin);
-            gtk_widget_show (view->details->popup_unpin);
+        if (!is_toplevel && !nemo_file_is_in_favorites (view->details->popup_file)) {
+            if (nemo_file_get_pinning (view->details->popup_file)) {
+                gtk_widget_hide (view->details->popup_pin);
+                gtk_widget_show (view->details->popup_unpin);
+            } else {
+                gtk_widget_show (view->details->popup_pin);
+                gtk_widget_hide (view->details->popup_unpin);
+            }
+
+            gtk_widget_show (view->details->popup_post_pin_separator);
         } else {
-            gtk_widget_show (view->details->popup_pin);
+            gtk_widget_hide (view->details->popup_pin);
             gtk_widget_hide (view->details->popup_unpin);
+            gtk_widget_hide (view->details->popup_post_pin_separator);
         }
 
         gboolean actions_visible = FALSE;
@@ -804,10 +826,10 @@ button_pressed_callback (GtkTreeView *treeview, GdkEventButton *event,
 
         for (l = view->details->action_items; l != NULL; l = l->next) {
             p = l->data;
-            if (nemo_action_get_visibility (p->action, tmp, parent, FALSE)) {
+            if (nemo_action_get_visibility (p->action, tmp, parent, FALSE, GTK_WINDOW (view->details->window))) {
                 gchar *action_label;
 
-                action_label = nemo_action_get_label (p->action, tmp, parent);
+                action_label = nemo_action_get_label (p->action, tmp, parent, GTK_WINDOW (view->details->window));
                 gtk_menu_item_set_label (GTK_MENU_ITEM (p->item), action_label);
                 g_free (action_label);
 
@@ -826,8 +848,8 @@ button_pressed_callback (GtkTreeView *treeview, GdkEventButton *event,
 				NULL, NULL, NULL, NULL,
 				event->button, event->time);
 
-		gtk_tree_view_set_cursor (view->details->tree_widget, cursor_path, NULL, FALSE);
-		gtk_tree_path_free (cursor_path);
+		gtk_tree_view_set_cursor (view->details->tree_widget, path, NULL, FALSE);
+        gtk_tree_path_free (path);
 
 		return FALSE;
 	} else if (event->button == 2 && event->type == GDK_BUTTON_PRESS) {
@@ -1206,7 +1228,7 @@ action_activated_callback (GtkMenuItem *item, ActionPayload *payload)
     GList *tmp = NULL;
     tmp = g_list_append (tmp, file);
 
-    nemo_action_activate (NEMO_ACTION (payload->action), tmp, parent);
+    nemo_action_activate (NEMO_ACTION (payload->action), tmp, parent, GTK_WINDOW (view->details->window));
 
     nemo_file_list_free (tmp);
 
@@ -1353,7 +1375,7 @@ create_popup_menu (FMTreeView *view)
 	gtk_menu_shell_append (GTK_MENU_SHELL (popup), menu_item);
 	view->details->popup_create_folder = menu_item;
 	
-	eel_gtk_menu_append_separator (GTK_MENU (popup));
+    view->details->popup_post_create_folder_separator = GTK_WIDGET (eel_gtk_menu_append_separator (GTK_MENU (popup)));
 	
 	/* add the "cut folder" menu item */
     menu_image = gtk_image_new_from_icon_name ("edit-cut-symbolic", GTK_ICON_SIZE_MENU);
@@ -1418,8 +1440,7 @@ create_popup_menu (FMTreeView *view)
     gtk_menu_shell_append (GTK_MENU_SHELL (popup), menu_item);
     view->details->popup_unpin = menu_item;
 
-
-    eel_gtk_menu_append_separator (GTK_MENU (popup));
+    view->details->popup_post_pin_separator = GTK_WIDGET (eel_gtk_menu_append_separator (GTK_MENU (popup)));
 
 	/* add the "move to trash" menu item */
 	menu_image = gtk_image_new_from_icon_name (NEMO_ICON_SYMBOLIC_TRASH_FULL,
@@ -1741,6 +1762,27 @@ sort_directories_first_changed_callback (gpointer callback_data)
                          compare_rows, view, NULL);
 }
 
+static void
+sort_favorites_first_changed_callback (gpointer callback_data)
+{
+    FMTreeView *view;
+    gboolean preference_value;
+
+    view = FM_TREE_VIEW (callback_data);
+
+    preference_value = g_settings_get_boolean (nemo_preferences,
+                                               NEMO_PREFERENCES_SORT_FAVORITES_FIRST);
+
+    if (preference_value != view->details->sort_favorites_first) {
+        view->details->sort_favorites_first = preference_value;
+    }
+
+    gtk_tree_model_sort_reset_default_sort_func (GTK_TREE_MODEL_SORT (view->details->sort_model));
+
+    gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (view->details->sort_model),
+                         compare_rows, view, NULL);
+}
+
 static void
 fm_tree_view_init (FMTreeView *view)
 {
@@ -1769,9 +1811,15 @@ fm_tree_view_init (FMTreeView *view)
     g_signal_connect_swapped (nemo_preferences,
                   "changed::" NEMO_PREFERENCES_SORT_DIRECTORIES_FIRST,
                   G_CALLBACK (sort_directories_first_changed_callback), view);
+                  
+    g_signal_connect_swapped (nemo_preferences,
+                  "changed::" NEMO_PREFERENCES_SORT_FAVORITES_FIRST,
+                  G_CALLBACK (sort_favorites_first_changed_callback), view);
 
     view->details->sort_directories_first = g_settings_get_boolean (nemo_preferences,
                                                                     NEMO_PREFERENCES_SORT_DIRECTORIES_FIRST);
+    view->details->sort_favorites_first = g_settings_get_boolean (nemo_preferences,
+                                                                    NEMO_PREFERENCES_SORT_FAVORITES_FIRST);
 
 	view->details->popup_file = NULL;
 
@@ -1864,6 +1912,10 @@ fm_tree_view_dispose (GObject *object)
     g_signal_handlers_disconnect_by_func (nemo_tree_sidebar_preferences,
                           G_CALLBACK(sort_directories_first_changed_callback),
                           view);
+    g_signal_handlers_disconnect_by_func (nemo_tree_sidebar_preferences,
+                          G_CALLBACK(sort_favorites_first_changed_callback),
+                          view);
+        
 
 	view->details->window = NULL;
 
